'From etoys2.2 of 1 October 2007 [latest update: #1675] on 7 December 2007 at 12:43:13 pm'!
"Change Set:		Games-Abalone
Date:			7 December 2007
Author:			Robert Krahn

Authors: Philipp Dobrigkeit, Peter Faymonville, and Tobias Flach

Hasso-Plattner-Institut
Software Architecture Group
http://www.hpi.uni-potsdam.de/swa/"!

RectangleMorph subclass: #AbaArrow
	instanceVariableNames: 'direction rotation move'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Games-Abalone'!

!AbaArrow commentStamp: '<historical>' prior: 0!
The AbaArrow Class is used to create and position the arrows on the game board. If an arrow is clicked by the user the selected spheres will move in the direction associated with the arrow. As a consequence each game board is provided with six arrow instances each representing a different moving direction. Instances are clickable and together with the spheres the only interface to the user's interaction with the game.

Copyright 2007: Philipp Dobrigkeit, Peter Faymonville, Tobias Flach
Released under MIT Licence
http://www.swa.hpi.uni-potsdam.de!

EllipseMorph subclass: #AbaField
	instanceVariableNames: 'sphere game id'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Games-Abalone'!

!AbaField commentStamp: '<historical>' prior: 0!
The AbaField Class is used to represent the fields on the game boards. Furthermore each field may contain a sphere and an arrow (both can co-exist). If a sphere wants to communicate with the game board it has to use the field as router (not needed for the arrows).

Copyright 2007: Philipp Dobrigkeit, Peter Faymonville, Tobias Flach
Released under MIT Licence
http://www.swa.hpi.uni-potsdam.de!

AbaField class
	instanceVariableNames: 'game'!
AlignmentMorph subclass: #AbaGameBoard
	instanceVariableNames: 'fields player1 player2 gameBoardFields activePlayer move shortestRow longestRow'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Games-Abalone'!

!AbaGameBoard commentStamp: '<historical>' prior: 0!
The AbaGameBoard class is the main class. It organizes the initialization and finalization of the game (including the creation of the board with fields, spheres and a score output) and acts as router for the communication between fields, players and the move instance. The class also manages the game flow (e.g. switching players).

Copyright 2007: Philipp Dobrigkeit, Peter Faymonville, Tobias Flach
Released under MIT Licence
http://www.swa.hpi.uni-potsdam.de!

Object subclass: #AbaMove
	instanceVariableNames: 'game activeFields moveWeight possibleMoves weightDir moveDirections arrows maxSpheres'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Games-Abalone'!

!AbaMove commentStamp: '<historical>' prior: 0!
The AbaMove Class contains the complete set of algorithms needed for the calculation of possible moves (and for executing them). It also acts as the arrow creator (which are plugged to the fields if moves are possible) and decides whether spheres can be highlighted/marked or not.

Copyright 2007: Philipp Dobrigkeit, Peter Faymonville, Tobias Flach
Released under MIT Licence
http://www.swa.hpi.uni-potsdam.de!

AlignmentMorph subclass: #AbaPlayer
	instanceVariableNames: 'name playerColor score game id spheres enemyColor position'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Games-Abalone'!

!AbaPlayer commentStamp: '<historical>' prior: 0!
The AbaPlayer class is represented as morph needed for the score output and for checking the game finished condition (one player has to push 6 enemy spheres out of bounds). The Spheres created are clickable in general but with no effect due to the assigned player (0).

Copyright 2007: Philipp Dobrigkeit, Peter Faymonville, Tobias Flach
Released under MIT Licence
http://www.swa.hpi.uni-potsdam.de!

EllipseMorph subclass: #AbaSphere
	instanceVariableNames: 'field selected player'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Games-Abalone'!

!AbaSphere commentStamp: '<historical>' prior: 0!
The AbaSphere class is used for creating and modifying a sphere which can be attached to a field on the game board. The class is provided with methods for a realistic shape and surface of the spheres. Instances are clickable and are (besides the arrows) the interface to the user's interaction with the game.

Copyright 2007: Philipp Dobrigkeit, Peter Faymonville, Tobias Flach
Released under MIT Licence
http://www.swa.hpi.uni-potsdam.de!


!AbaArrow methodsFor: 'initialization' stamp: 'tfl 1/31/2007 16:52'!
attachArrowPicture: aPicture 
	"attaches the arrow picture to the button so that it actually
	looks like an arrow"
	| defaultLabel |
	(defaultLabel := self findA: SketchMorph)
		ifNotNil: [defaultLabel delete].
	self extent: aPicture extent + (borderWidth + 6).
	aPicture position: self center - (aPicture extent // 2).
	self addMorph: aPicture.
	arrow := aPicture! !

!AbaArrow methodsFor: 'initialization' stamp: 'tfl 1/31/2007 16:52'!
createArrowPicture
	"creates the arrow picture for the arrow button and returns the
	picture morph to the sender"
	| picture |
	picture := PolygonMorph arrowPrototype.
	picture arrowLength: 30;
		 arrowSpec: 7 @ 4;
		 rotationDegrees: rotation;
		 borderColor: Color red.
	^ picture! !

!AbaArrow methodsFor: 'initialization' stamp: 'rkrk 11/22/2007 21:44'!
direction: aDirection 
	"for a given direction a rotation message is sent to the arrow
	image so that the arrow points in the given direction (the
	rotation values are predefined for each direction depending
	on the game board's alignment, the dictionary is defined
	temporarily so that the arrows only know the possible rotation
	values (the keys depend on the the number of fields on the
	game board which can be determined by the longest row)"
	| rotationCol longestRow |
	longestRow := move longestRow.
	rotationCol := Dictionary new.
	rotationCol add: (longestRow negated - 1) -> 330; "upLeft"
		 add: (longestRow negated) -> 34; "upRight"
		 add: 1 -> 92; "Right"
		 add: (longestRow + 1) -> 150; "downRight"
		 add: longestRow -> 214; "downLeft"
		 add: -1 -> 272. "Left" 		 
	direction := aDirection.
	self
		rotation: (rotationCol at: aDirection)! !

!AbaArrow methodsFor: 'initialization' stamp: 'pd 1/12/2007 18:37'!
initialize
	"initializes a new arrow with a given direction with a reaction
	on a mouse click event"
	super initialize.
	field := false.
	self color: Color transparent;
		 borderColor: Color transparent;
		
		on: #mouseUp
		send: #mouseUp:
		to: self! !

!AbaArrow methodsFor: 'initialization' stamp: 'tfl 1/31/2007 16:52'!
rotation: aDegree 
	"draws a rotated arrow on the button"
	rotation := aDegree.
	self attachArrowPicture: self createArrowPicture! !

!AbaArrow methodsFor: 'interaction' stamp: 'PF 2/1/2007 22:44'!
mouseUp: anEvent 
	"if the sender (an arrow button) is enabled the highlighted
	spheres are moved on the click. starts move execution."
	move startMove: direction! !

!AbaArrow methodsFor: 'interaction' stamp: 'PF 2/1/2007 22:42'!
removeFromField
"gets called before a new Move is calculated or when move is executed. Removes Arrow from its parentMorph (Field) without the need of a reference"
	self delete! !

!AbaArrow methodsFor: 'interaction' stamp: 'pd 1/12/2007 18:33'!
resetPosition: aPoint 
	"for redraw purposes, align with field"
	self center: aPoint! !

!AbaArrow methodsFor: 'accessing' stamp: 'pd 1/12/2007 17:18'!
move: anAbaMove
	move := anAbaMove! !


!AbaField methodsFor: 'interaction' stamp: 'pd 1/22/2007 20:26'!
addArrow: anArrow 
	"adds a given arrow to the field"
	| center |
	center := self center.
	self addMorph: anArrow.
	anArrow 
		 resetPosition: center! !

!AbaField methodsFor: 'interaction' stamp: 'pd 1/9/2007 01:23'!
addSphere: aSphere 
	"adds a given sphere to the field"
	| center |
	center := self center.
	sphere := aSphere.
	self addMorph: aSphere.
	sphere field: self;
		 resetPosition: center! !

!AbaField methodsFor: 'interaction' stamp: 'tfl 1/6/2007 19:25'!
checkHighlight: aPlayerId
	"forwards the request to the game board instance which has to decide whether the field can be highlighted"
  
	^game checkHighlight: self id player: aPlayerId! !

!AbaField methodsFor: 'interaction' stamp: 'PF 2/1/2007 22:46'!
highlight
	"the containing sphere was clicked and gets now a green border to
	indicate the highlight"
	self borderColor: Color green.! !

!AbaField methodsFor: 'interaction' stamp: 'PF 2/1/2007 22:50'!
removeHighlight
	"removes the highlight from a field if it is not the connection
	between two highlighted fields (the remove request sent to
	the game board instance will then return false"
	(game removeHighlight: id)
		ifTrue: [self resetHighlight.
			^ true].
	^ false! !

!AbaField methodsFor: 'interaction' stamp: 'pd 1/9/2007 01:29'!
removeSphere
	|tempShere|
	"removes a sphere from a field"
	self removeMorph: sphere.
	tempShere := sphere.
	sphere := false.
	^ tempShere! !

!AbaField methodsFor: 'interaction' stamp: 'pd 1/13/2007 01:38'!
resetHighlight
	"this field is no longer active"
	self borderColor: Color transparent! !

!AbaField methodsFor: 'interaction' stamp: 'PF 2/1/2007 22:51'!
sphereDeselect
	"removes sphere from the activeFields and makes it remove its highlight"
	sphere resetHighlight.
	self resetHighlight! !

!AbaField methodsFor: 'accessing' stamp: 'PF 2/1/2007 22:45'!
fieldOwner
	"returns whether the field is empty or the id of the player"
	sphere = false
		ifTrue: [^ false].
	^ sphere getPlayer! !

!AbaField methodsFor: 'accessing' stamp: 'pd 1/5/2007 01:00'!
game: aGameBoard

	game := aGameBoard.! !

!AbaField methodsFor: 'accessing' stamp: 'pd 1/6/2007 00:27'!
id
	^id! !

!AbaField methodsFor: 'accessing' stamp: 'pd 1/6/2007 00:25'!
id: anId
	id := anId! !

!AbaField methodsFor: 'accessing' stamp: 'pd 1/8/2007 23:05'!
sphere
^sphere! !

!AbaField methodsFor: 'initialization' stamp: 'PF 2/1/2007 22:47'!
initialize
"initially, a field doesn't contain a sphere and has no highlight (transparent border)"
	super initialize.
	sphere := false.
	self color: Color transparent;
		 borderWidth: 1;
		 borderColor: Color transparent;
		 extent: 48 @ 48! !

!AbaField methodsFor: 'initialization' stamp: 'PF 2/1/2007 22:49'!
makeGradient
	"give the fields a round look, so they actually look like a natural pit"
	| fill |
	fill := GradientFillStyle ramp: {0.0 -> self color. 1.0 -> self color darker}.
	fill origin: 30 @ 30.
	fill direction: 12 @ 12.
	fill radial: true.
	self addMorph: (EllipseMorph new color: self color;
			 borderWidth: 0;
			 top: 2;
			 left: 2;
			 extent: 44 @ 44;
			 fillStyle: fill)! !


!AbaGameBoard methodsFor: 'interaction' stamp: 'pd 1/12/2007 18:31'!
addArrowToField: anArrow field: aFieldId
	(fields at: aFieldId) addArrow: anArrow.! !

!AbaGameBoard methodsFor: 'interaction' stamp: 'tfl 1/31/2007 16:06'!
checkField: anId 
	"returns the state for the given field (#out: field is out of
	bounds, #empty: no sphere on this field, #enemy/#own:
	enemy/own sphere on this field)"
	| field |
	(gameBoardFields includes: anId)
		ifFalse: [^#out].
	field := fields at: anId.
	field fieldOwner = false
		ifTrue: [^#empty].
	field fieldOwner = activePlayer
		ifTrue: [^#own]
		ifFalse: [^#enemy]! !

!AbaGameBoard methodsFor: 'interaction' stamp: 'pd 1/29/2007 12:09'!
checkHighlight: aFieldId player: aPlayerId 
	"checks whether the given player is the active one and
	whether the given field can be connected with currently
	highlighted/active fields"
	activePlayer = aPlayerId
		ifFalse: [^ false].
	(move checkActiveFields: aFieldId)
		ifFalse: [^ false].
	move checkMoves.
	^ true! !

!AbaGameBoard methodsFor: 'interaction' stamp: 'pd 1/29/2007 11:40'!
removeHighlight: aFieldId 
	"removes the given field from the list of active fields if it is not
	the connection of two highlighted/active fields"
	^ move removeField: aFieldId! !

!AbaGameBoard methodsFor: 'move' stamp: 'tfl 1/9/2007 23:16'!
addPoint: anEnemyPlayer
	"adds a point to the player's score (if an enemy sphere moved out of bounds)"
	
	anEnemyPlayer = 1
		ifTrue: [player2 addPoint]
		ifFalse: [player1 addPoint]
! !

!AbaGameBoard methodsFor: 'move' stamp: 'pd 2/9/2007 20:27'!
finish: aPlayer 
	"this method is executed if one of the players reaches the
	target score of 6 points"
	aPlayer removeHighlight.
	activePlayer := 0! !

!AbaGameBoard methodsFor: 'move' stamp: 'PF 2/1/2007 23:00'!
moveSphere: anAssociation 
	"attaches the sphere from a given field association to the new field (if the destination field is out of bounds - add a point to the players score)"

	| sphere |
	sphere := (fields at: anAssociation key) removeSphere.
	(gameBoardFields includes: anAssociation value)
		ifTrue: [(fields at: anAssociation value)
				addSphere: sphere.
			^ self].
	self addPoint: sphere getPlayer! !

!AbaGameBoard methodsFor: 'move' stamp: 'pd 1/22/2007 20:31'!
sphereDeselect: aFieldId 
	"remove highlight from a field"
	(fields at: aFieldId) sphereDeselect! !

!AbaGameBoard methodsFor: 'move' stamp: 'pd 2/9/2007 20:26'!
switchPlayers
	"make the other player active"
	activePlayer = 0 ifTrue: [^self].
	activePlayer = 1
		ifTrue: [player1 removeHighlight.
			activePlayer := 2.
			player2 highlight.
			^ self]
		ifFalse: [player2 removeHighlight.
			activePlayer := 1.
			player1 highlight.
			^ self]! !

!AbaGameBoard methodsFor: 'creation' stamp: 'tfl 2/1/2007 16:59'!
calculateGameBoardFields
	"calculates the id's which will be valid field id's on the game
	board depending on the shortest/longestRow values which
	were defined in the initialize method (the number of
	fields per rows increments/decrements always by 1)"
	| fieldCollection currentId |
	fieldCollection := SortedCollection new.
	currentId := 0.
	"adds the valid id's for the upper rows till center row to the
	collection"
	shortestRow
		to: longestRow
		do: [:each | 
			1
				to: each
				do: [:bias | 
					fieldCollection add: currentId.
					currentId := currentId + 1].
			currentId := currentId + (longestRow - each)].
	"adds the valid id's for the lower rows (the calculation is in reversed order due to the scheme of valid fields in the array)"
	longestRow - 1
		to: shortestRow
		by: -1
		do: [:each | 
			currentId := currentId + (longestRow - each).
			1
				to: each
				do: [:bias | 
					fieldCollection add: currentId.
					currentId := currentId + 1]].
	self gameBoardFields: fieldCollection! !

!AbaGameBoard methodsFor: 'creation' stamp: 'tfl 1/5/2007 21:30'!
columnForRows
	"creates a new column for rows"

	^ AlignmentMorph newColumn color: color! !

!AbaGameBoard methodsFor: 'creation' stamp: 'pd 1/13/2007 01:35'!
createField: anId 
	"creates a new empty field with a given id (stored in the
	dictionary) and adds it to the current row"
	| morph |
	morph := AbaField new game: self;
				 id: anId;
				color: self color;makeGradient.
	fields add: anId -> morph.
	^ self rowForFields addMorph: morph! !

!AbaGameBoard methodsFor: 'creation' stamp: 'PF 2/1/2007 22:53'!
createGameBoard
	"creates a new game board using the id's of the game board
	fields for separating fields which are in different rows (for centering
	purposes)"
	| rowCollection column |
	rowCollection := OrderedCollection new.
	column := self columnForRows.
	1
		to: longestRow
		do: [:each | rowCollection
				add: (gameBoardFields
						select: [:element | element >= (each - 1 * longestRow) & (element < (each * longestRow))])].
	rowCollection
		do: [:each | column
				addMorphBack: (self createRowFields: each)].
	column addMorphBack: self createScoreOutput.
	self addMorphBack: column! !

!AbaGameBoard methodsFor: 'creation' stamp: 'PF 2/1/2007 22:53'!
createPlayers
	"creates two instances of the Player Class"
	| firstColor secondColor firstName secondName|
	firstColor := Color white.
	firstName:= 'White'.
	secondColor := Color black.
	secondName:= 'Black'.
	player1 := AbaPlayer new id: 1;
				 playerColor: firstColor;
				 enemyColor: secondColor;
				 color: self defaultColor;
				 name: firstName;
				 game: self;
				 createMorph: #left.
	player2 := AbaPlayer new id: 2;
				 playerColor: secondColor;
				 enemyColor: firstColor;
				 color: self defaultColor;
				 name: secondName;
				 game: self;
				 createMorph: #right! !

!AbaGameBoard methodsFor: 'creation' stamp: 'PF 2/1/2007 22:54'!
createRowFields: aCollection 
	"creates fields from a given collection and adds them to a row"
	| row |
	row := self rowForFields.
	aCollection
		do: [:each | row
				addMorphBack: (self createField: each)].
	^ row! !

!AbaGameBoard methodsFor: 'creation' stamp: 'PF 2/1/2007 22:54'!
createScoreOutput
	"attaches the player objects to the gameboard morph"
	| scoreRow |
	scoreRow := self rowForFields.
	scoreRow
		addMorphBack: (player1).
	scoreRow
		addMorphBack: (player2).
	^ scoreRow! !

!AbaGameBoard methodsFor: 'creation' stamp: 'PF 2/1/2007 22:55'!
createSphere: anId player: aPlayer 
	"creates a sphere with a given color at a given field"
	| morph field |
	field := fields at: anId.
	morph := AbaSphere new color: aPlayer playerColor;
				 setPlayer: aPlayer id;
				 makeGradient.
	field addSphere: morph! !

!AbaGameBoard methodsFor: 'creation' stamp: 'tfl 2/1/2007 17:05'!
createStartPositions
	"set the spheres for each player in initial fields"
	(self getStartFields: 1)
		do: [:each | self createSphere: each player: player1].
	(self getStartFields: 2)
		do: [:each | self createSphere: each player: player2]! !

!AbaGameBoard methodsFor: 'creation' stamp: 'PF 2/2/2007 00:27'!
getStartFields: anId 
	"returns the field id's where the spheres for the given player
	(identified by the id) will be
	positioned at the beginning of the game - the playerfields
	collection can be modified for different
	starting position, the numbers are absolute values beginning
	with 1 and are incremented by 1 for each field (going row by
	row), e.g. the first row starts with 1, the second with 6 (if the
	shortest row is 5 fields long) - these values are different from
	the final field id's"
	| idCollection playerfields sum |
	idCollection := SortedCollection new.
	playerfields := SortedCollection new.
	sum := 0.
	"depending on the actual size of the gameboard we calculate which fields must be getting a 	sphere. the opposite player is getting a symmetrical position from the end of the board"
	0 to: (longestRow - shortestRow - 3) do: [:each | sum := sum + shortestRow + each].
	1 to: sum do: [:each | playerfields add: each].
	sum := sum + 1 + ((shortestRow -1) / 2) floor.
	(sum) to: (sum + shortestRow - 3) do: [:each | playerfields add: each].

	anId = 1
		ifTrue: [playerfields
				do: [:each | idCollection
						add: (gameBoardFields at: each)].
			^ idCollection]
		ifFalse: [playerfields
				do: [:each | idCollection
						add: (gameBoardFields at: ((gameBoardFields size) + 1 - each)).]. 
			^ idCollection]! !

!AbaGameBoard methodsFor: 'creation' stamp: 'pd 1/13/2007 01:43'!
rowForFields
	"creates a new alignment morph for a centered row of fields"
	^ AlignmentMorph newRow color: color;
		 cellInset: 10;
		 borderWidth: 0;
		 layoutInset: 2;
		 vResizing: #shrinkWrap;
		 hResizing: #shrinkWrap;
		 wrapCentering: #center! !

!AbaGameBoard methodsFor: 'initialization' stamp: 'pd 1/17/2007 11:08'!
defaultColor
	"answer the default color/fill style for the receiver"
	^ Color tan! !

!AbaGameBoard methodsFor: 'initialization' stamp: 'pd 2/6/2007 16:07'!
initGame
	"used to initialize or reset the game to starting conditions, the bottom player always begins"
	self removeAllMorphs.
	fields := Dictionary new.
	move := AbaMove new.
	move game: self; initializeMoveDirections;
		 createArrows; maxSpheres: (shortestRow - 2).
	self createPlayers; createGameBoard; createStartPositions.
	activePlayer := 2.
	player2 highlight! !

!AbaGameBoard methodsFor: 'initialization' stamp: 'pd 2/6/2007 16:15'!
initGame: aSize
	"used to initialize or reset the game to starting conditions, the bottom player always begins"
	self setRowLength: aSize; calculateGameBoardFields; initGame! !

!AbaGameBoard methodsFor: 'initialization' stamp: 'pd 2/6/2007 16:13'!
initialize
	"set starting values and call the initGame method which draws
	the content, the shortest and longest row define the
	dimension of the game board (by default these values are set
	to the properties of the original abalone game, shortest: 5 and
	longest: 9)"
	super initialize.
	self vResizing: #shrinkWrap;
		 hResizing: #shrinkWrap;
		 useRoundedCorners;
		 layoutInset: 10;
		 borderColor: self color muchDarker;
		 borderWidth: 10.
	self setRowLength: 5;
		 calculateGameBoardFields. 
	self initGame! !

!AbaGameBoard methodsFor: 'initialization' stamp: 'pd 2/6/2007 16:13'!
setRowLength: aLength
	self shortestRow: aLength; longestRow: (aLength * 2 - 1)! !

!AbaGameBoard methodsFor: 'accessing' stamp: 'pd 1/11/2007 18:57'!
gameBoardFields
^gameBoardFields! !

!AbaGameBoard methodsFor: 'accessing' stamp: 'tfl 2/1/2007 16:11'!
gameBoardFields: aCollection
	gameBoardFields:= aCollection.! !

!AbaGameBoard methodsFor: 'accessing' stamp: 'pd 2/6/2007 16:11'!
longestRow
	^ longestRow! !

!AbaGameBoard methodsFor: 'accessing' stamp: 'pd 2/6/2007 16:12'!
longestRow: aLength 
	longestRow := aLength! !

!AbaGameBoard methodsFor: 'accessing' stamp: 'tfl 2/1/2007 16:26'!
shortestRow
	^ shortestRow! !

!AbaGameBoard methodsFor: 'accessing' stamp: 'pd 2/6/2007 16:12'!
shortestRow: aLength 
	shortestRow := aLength! !


!AbaGameBoard class methodsFor: 'as yet unclassified' stamp: 'pd 2/9/2007 20:03'!
descriptionForPartsBin
	^ self partName:	'Abalone'
		categories:		#('Games')
		documentation:	'Win by pushing Spheres of other player out of bounds.


Copyright 2007: Philipp Dobrigkeit, Peter Faymonville, Tobias Flach
Released under MIT Licence
http://www.swa.hpi.uni-potsdam.de'! !


!AbaMove methodsFor: 'accessing' stamp: 'pd 1/11/2007 19:21'!
activeFields
^activeFields! !

!AbaMove methodsFor: 'accessing' stamp: 'pd 1/8/2007 16:39'!
game: aGame
	game := aGame! !

!AbaMove methodsFor: 'accessing' stamp: 'PF 2/2/2007 00:20'!
maxSpheres: anInt
 maxSpheres := anInt! !

!AbaMove methodsFor: 'accessing' stamp: 'pd 1/11/2007 18:47'!
moveDirections
	^moveDirections! !

!AbaMove methodsFor: 'accessing' stamp: 'pd 1/8/2007 22:46'!
moves
^possibleMoves! !

!AbaMove methodsFor: 'moveCalculation' stamp: 'pd 1/11/2007 19:17'!
calculateMoveDir: aDirection weight: aWeight 
	"Given a direction and a weight of spheres, check if move is
	possible "
	aDirection = weightDir
		ifTrue: [^ self
				checkFieldPath: activeFields min
				direction: aDirection
				weight: aWeight].
	aDirection = (weightDir * -1)
		ifTrue: [^ self
				checkFieldPath: activeFields max
				direction: aDirection
				weight: aWeight].
	activeFields
		do: [:each | (self
					checkFieldPath: each
					direction: aDirection
					weight: aWeight)
				ifFalse: [^ false]].
	^ true! !

!AbaMove methodsFor: 'moveCalculation' stamp: 'PF 2/1/2007 23:02'!
calculateMoves
	"on the basis of the weights for all directions this method checks in which directions moves are possible and stores the information in a dictionary"

	possibleMoves := Dictionary new.
	moveWeight
		associationsDo: [:each | possibleMoves add: each key
					-> (self calculateMoveDir: each key weight: each value)]! !

!AbaMove methodsFor: 'moveCalculation' stamp: 'PF 2/1/2007 23:02'!
calculateMoveWeights
	"creates a dictionary where all directions are associated with
	the move weight in this direction
	if more than one sphere is selected the two directions
	(forward/backward) in which the spheres are aligned get a
	weight equal to the number of selected spheres"
	moveWeight := Dictionary new.
	moveDirections
		do: [:each | moveWeight add: each -> 1].
	weightDir := -1.
	"more than 2 spheres in a row, may push less opponent spheres back"
	activeFields size > 1
		ifTrue: [weightDir := (activeFields at: 1)
						- (activeFields at: 2).
			moveWeight at: weightDir put: activeFields size.
			moveWeight at: weightDir negated put: activeFields size]! !

!AbaMove methodsFor: 'moveCalculation' stamp: 'pd 1/11/2007 18:53'!
checkActiveFields: aFieldId 
	"adds the given field to the list of highlighted/active fields if
	the list is empty or the given field builds a row with already
	highlighted/active fields"
	activeFields
		ifEmpty: [activeFields add: aFieldId.
			^ true].
	activeFields add: aFieldId.
	self compareActiveFields
		ifTrue: [^ true].
	activeFields remove: aFieldId.
	^ false! !

!AbaMove methodsFor: 'moveCalculation' stamp: 'tfl 1/31/2007 16:03'!
checkFieldPath: anId direction: aDirection weight: aWeight 
	"For a given field, direction and weight of spheres check if it is
	possible to move on the board (the checkField method will
	return #out, #empty, #enemy or #own depending on the
	field's state - if checkField returns #enemy a move can be
	possible if the weight is high enough and the last important
	(or earlier) field in moving direction is empty or out of
	bounds); return false if the player would move one of its own spheres out of bounds"
	| fieldCheck |
	(self checkSuicideMove: anId direction: aDirection)
		ifTrue: [^ false].
	1
		to: aWeight
		do: [:each | 
			fieldCheck := game checkField: anId + (each * aDirection).
			fieldCheck = #out
				ifTrue: [^ true].
			fieldCheck = #empty
				ifTrue: [^ true].
			fieldCheck = #own
				ifTrue: [^ false]].
	^ false! !

!AbaMove methodsFor: 'moveCalculation' stamp: 'PF 2/1/2007 23:05'!
checkMoves
	"starts to calculate which moves can be made"
	activeFields size = 0
		ifTrue: [self resetMoves; drawArrows.
			^ self].
	self calculateMoveWeights; calculateMoves; drawArrows! !

!AbaMove methodsFor: 'moveCalculation' stamp: 'pd 1/11/2007 18:57'!
checkSuicideMove: anId direction: aDirection 
	"checks whether the player would move an own sphere out of
	bounds if moving in the given direction"
	((game gameBoardFields) includes: anId + aDirection)
		ifFalse: [^ true].
	^ false! !

!AbaMove methodsFor: 'moveCalculation' stamp: 'tfl 2/1/2007 20:08'!
compareActiveFields
	"checks whether the elements in the active fields list are
	neighbors and in a row"
	| difference |
	activeFields size < 2
		ifTrue: [^ true].
	activeFields size > maxSpheres
		ifTrue: [^ false].
	difference := (activeFields at: 1)
				- (activeFields at: 2).
	(moveDirections includes: difference)
		ifTrue: [1
				to: activeFields size - 1
				do: [:each | 
					(activeFields at: each)
							- (activeFields at: each + 1) = difference
						ifFalse: [^ false]].
			^ true].
	^ false! !

!AbaMove methodsFor: 'creation' stamp: 'rkrk 11/22/2007 21:20'!
createArrows
	moveDirections do: [:each | arrows at: each put: (self createArrow: each). ]! !

!AbaMove methodsFor: 'creation' stamp: 'rkrk 11/22/2007 21:36'!
createArrow: aDirection 
	"creates a new arrow with a given direction (acts
	simultaniously as an id stored in the arrows
	dictionary) and adds it to the current row"

	^ AbaArrow new
		move: self;
		direction: aDirection;
		yourself! !

!AbaMove methodsFor: 'interaction' stamp: 'rkrk 11/22/2007 21:31'!
drawArrows
	"draws arrows if moves in their direction are possible, first: arrow in main direction and his direct neighbors (e.g. the neighbors of the #upperRight arrow are the #upperLeft and the #right arrow), second: arrows in opposite main direction and his direct neighbors"
	| weightDirKey arrowOffset |
	arrows
		do: [:each | each removeFromField].
	weightDirKey := moveDirections keyAtValue: weightDir.
	-1
		to: 1
		do: [:each | 
			arrowOffset := moveDirections at: weightDirKey + each \\ 6.
			"check arrows in main direction"
			(possibleMoves at: arrowOffset)
				ifTrue: [self drawArrow: activeFields min offset: arrowOffset].
			"check arrows in opposite direction"
			(possibleMoves at: arrowOffset * -1)
				ifTrue: [self drawArrow: activeFields max offset: arrowOffset * -1]]! !

!AbaMove methodsFor: 'interaction' stamp: 'pd 1/12/2007 18:45'!
drawArrow: aFieldId offset: anOffset 
	"tell a field to draw an arrow"
	game
		addArrowToField: (arrows at: anOffset)
		field: aFieldId + anOffset! !

!AbaMove methodsFor: 'interaction' stamp: 'tfl 2/1/2007 17:28'!
longestRow
	^ game longestRow! !

!AbaMove methodsFor: 'interaction' stamp: 'PF 2/1/2007 23:09'!
removeField: aFieldId 
	"removes a field from the activeFields and recalculates the possible moves"
	activeFields remove: aFieldId.
	self compareActiveFields
		ifTrue: [self checkMoves.
			^ true]
		ifFalse: [activeFields add: aFieldId.
			^ false]! !

!AbaMove methodsFor: 'interaction' stamp: 'pd 1/11/2007 23:28'!
resetMoves
	"remove all arrows for movement"
	activeFields := SortedCollection new.
	moveDirections do: [:each | possibleMoves add: each->false]! !

!AbaMove methodsFor: 'move' stamp: 'PF 2/1/2007 23:06'!
executeMove: aDirection 
	"calculate which spheres need to be moved and executes"
	| aWeight |
	aWeight := moveWeight at: aDirection.
	aDirection = weightDir
		ifTrue: [self
				moveSpheres: (self
						getTargetField: activeFields min
						direction: aDirection
						weight: aWeight)
				direction: aDirection
				end: activeFields max.
			^ true].
	aDirection = (weightDir * -1)
		ifTrue: [self
				moveSpheres: (self
						getTargetField: activeFields max
						direction: aDirection
						weight: aWeight)
				direction: aDirection
				end: activeFields min.
			^ true].
	activeFields
		do: [:each | self moveSphere: each -> (each + aDirection)].
	^ true! !

!AbaMove methodsFor: 'move' stamp: 'PF 2/1/2007 23:07'!
getTargetField: anId direction: aDirection weight: aWeight 
	"For a move calculate to which field the spheres are moved,
	including opponent spheres so the first exchange is always with the empty field or out of bounds"
	| fieldCheck |
	1
		to: aWeight
		do: [:each | 
			fieldCheck := game checkField: anId + (each * aDirection).
			(fieldCheck = #empty | (fieldCheck = #out))
				ifTrue: [^ anId + (each * aDirection)]]! !

!AbaMove methodsFor: 'move' stamp: 'pd 1/9/2007 02:01'!
moveSpheres: aTargetField direction: aDirection end: anEndField 
	"spheres are in a row, starting from the empty field move spheres forward, including possible opposing spheres"
	1
		to: aTargetField - anEndField / aDirection
		do: [:each | self moveSphere: aTargetField - (each * aDirection) -> (aTargetField - (each - 1 * aDirection))]! !

!AbaMove methodsFor: 'move' stamp: 'pd 1/9/2007 12:18'!
moveSphere: anAssociation 
	"actually exchange spheres with a empty field"
	game moveSphere: anAssociation! !

!AbaMove methodsFor: 'move' stamp: 'PF 2/1/2007 23:11'!
startMove: aDirection 
	"reset all move conditions and exchange spheres. after this method the other player is active"
	activeFields
		do: [:each | 
			game sphereDeselect:each].
	self executeMove: aDirection;
		 resetMoves;
		 checkMoves.
	game switchPlayers
! !

!AbaMove methodsFor: 'initialization' stamp: 'PF 2/1/2007 23:08'!
initialize
	"active fields is needed to determine which fields are
	currently highlighted - the arrows dictionary will be used to
	store the arrow morphs
	maxSpheres declares how many spheres can be selected and moved (default abalone rule is 3)"
	activeFields := SortedCollection new.
	maxSpheres := 3.
	arrows := Dictionary new! !

!AbaMove methodsFor: 'initialization' stamp: 'rkrk 11/22/2007 21:37'!
initializeMoveDirections
	"moveDirections contains the moving directions for the
	possible arrow id's (the direction values depend on the game
	boards size (default: 9)"
	| longestRow |
	longestRow := game longestRow.
	moveDirections := Dictionary new.

	moveDirections add: 0 -> (longestRow negated - 1); "upLeft"
		 add: 1 -> longestRow negated; "upRight"
		 add: 2 -> 1; "Right"
		 add: 3 -> (longestRow + 1); "downRight"
		 add: 4 -> longestRow; "downLeft"
		 add: 5 -> -1 "Left"! !


!AbaMove class methodsFor: 'as yet unclassified' stamp: 'pd 1/11/2007 19:08'!
new
	^ super new initialize! !


!AbaPlayer methodsFor: 'scoring' stamp: 'tfl 1/31/2007 17:06'!
addPoint
	"adds a points to the player's score and makes an enemy
	sphere visible on the player's score board (the border is
	removed because of missing interpolation between border
	and sphere color)"
	score := score + 1.
	(spheres at: score) borderWidth: 0;
		 color: enemyColor;
		 makeGradient.
	self checkScore! !

!AbaPlayer methodsFor: 'scoring' stamp: 'tfl 1/31/2007 17:04'!
addSphere: aSphere score: aScore 
	"connects the sphere with a player which becomes visible
	if the player reaches the given score"
	spheres at: aScore put: aSphere! !

!AbaPlayer methodsFor: 'scoring' stamp: 'pd 2/9/2007 20:30'!
checkScore
	"checks whether the player's score reached the target of 6
	points"
	score = 6
		ifTrue: [self finish.
			game finish: self]! !

!AbaPlayer methodsFor: 'scoring' stamp: 'pd 2/9/2007 20:20'!
finish
	"prints a win message on the score output on the game board"
	| text |
	text := TextMorph new
				string: self name , ' wins'
				fontName: 'KomikaText'
				size: 30;
				 backgroundColor: Color white;
				 borderColor: Color black;
				 borderWidth: 1;
				 extent: 150 @ 30;
				 centered.
	position = #left
		ifTrue: [self
				addMorphBack: (self rowForFields addMorphBack: text)]
		ifFalse: [self
				addMorph: (self rowForFields addMorphBack: text)]! !

!AbaPlayer methodsFor: 'creation' stamp: 'PF 2/1/2007 23:12'!
createMorph: aPosition 
	"creates a morph for a given position (#left or #right) containing
	the player's name and the output for the spheres sacked (position of name and spheres is reversed on the opposite site"
	position := aPosition.
	self cellInset: 10;
		 borderWidth: 0;
		 borderColor: playerColor;
		 layoutInset: 2;
		 vResizing: #shrinkWrap;
		 hResizing: #shrinkWrap;
		 useRoundedCorners;
		 wrapCentering: #center.
	position = #left
		ifTrue: [self addMorphBack: self createName].
	self
		addMorphBack: (self createSpheres: position).
	position = #right
		ifTrue: [self addMorphBack: self createName]! !

!AbaPlayer methodsFor: 'creation' stamp: 'pd 2/9/2007 20:23'!
createName
	"create the morph with the player name"
	^ self rowForFields addMorphBack: (TextMorph new
			string: name
			fontName: 'KomikaText'
			size: 22;
			 color: playerColor;
			 backgroundColor: Color transparent)! !

!AbaPlayer methodsFor: 'creation' stamp: 'tfl 1/31/2007 17:18'!
createSpheres: aPosition 
	"creates the spheres for the score output (they become visible
	when an enemy sphere is moving out of bounds) and adds them to a row for proper alignment"
	| sphereRow sphere |
	sphereRow := self rowForFields.
	1
		to: 6
		do: [:each | 
			sphere := AbaSphere new color: Color transparent;
						 setPlayer: 0;
						 extent: 15 @ 15;
						 borderColor: self playerColor;
						 borderWidth: 1.
			self addSphere: sphere score: each.
			aPosition = #left
				ifTrue: [sphereRow addMorph: sphere]
				ifFalse: [sphereRow addMorphBack: sphere]].
	^ sphereRow! !

!AbaPlayer methodsFor: 'creation' stamp: 'pd 1/29/2007 11:48'!
rowForFields
	"creates a new alignment morph for a centered row of fields"
	^ AlignmentMorph newRow color: Color transparent;
		 cellInset: 10;
		 borderWidth: 0;
		 layoutInset: 2;
		 vResizing: #shrinkWrap;
		 hResizing: #shrinkWrap;
		 wrapCentering: #center! !

!AbaPlayer methodsFor: 'accessing' stamp: 'tfl 1/31/2007 16:56'!
enemyColor
	^ enemyColor! !

!AbaPlayer methodsFor: 'accessing' stamp: 'tfl 1/31/2007 16:56'!
enemyColor: aColor 
	enemyColor := aColor! !

!AbaPlayer methodsFor: 'accessing' stamp: 'tfl 1/9/2007 19:15'!
game: aGameBoard

	game:= aGameBoard! !

!AbaPlayer methodsFor: 'accessing' stamp: 'pd 1/11/2007 18:08'!
id

	^id! !

!AbaPlayer methodsFor: 'accessing' stamp: 'tfl 1/9/2007 23:12'!
id: anId

	id:= anId! !

!AbaPlayer methodsFor: 'accessing' stamp: 'pd 1/11/2007 18:23'!
name
	^name! !

!AbaPlayer methodsFor: 'accessing' stamp: 'pd 1/11/2007 18:19'!
name: aName
	name := aName! !

!AbaPlayer methodsFor: 'accessing' stamp: 'pd 1/29/2007 12:01'!
playerColor
	^ playerColor! !

!AbaPlayer methodsFor: 'accessing' stamp: 'pd 1/29/2007 12:01'!
playerColor: aColor 
	playerColor := aColor! !

!AbaPlayer methodsFor: 'interaction' stamp: 'PF 2/1/2007 23:14'!
highlight
	"this player is active"
	self borderWidth: 1! !

!AbaPlayer methodsFor: 'interaction' stamp: 'pd 1/29/2007 12:08'!
removeHighlight
	self borderWidth: 0! !

!AbaPlayer methodsFor: 'initialize' stamp: 'pd 1/29/2007 11:58'!
initialize
	super initialize.

	score := 0.
	spheres := Dictionary new.
! !


!AbaSphere methodsFor: 'interaction' stamp: 'tfl 1/24/2007 17:28'!
aMouseUp: anEvent 
	"sends a select/deselect request for the field to the sphere attached if mouse has been clicked (if possible)"	

	(self getPlayer < 1) ifTrue: [^self].
	selected
		ifTrue: [self deselect.
			^ self]
		ifFalse: [self select.
			^ self]! !

!AbaSphere methodsFor: 'interaction' stamp: 'pd 1/9/2007 02:03'!
deselect
	"remove the sphere from the selected ones"
	field removeHighlight
		ifTrue: [selected := false]! !

!AbaSphere methodsFor: 'interaction' stamp: 'pd 1/8/2007 23:24'!
resetHighlight
selected := false! !

!AbaSphere methodsFor: 'interaction' stamp: 'pd 1/12/2007 16:17'!
resetPosition: aPoint 
	"for redraw purposes, align with field"
	self center: aPoint! !

!AbaSphere methodsFor: 'interaction' stamp: 'pd 1/11/2007 18:34'!
select
	"selects the field the sphere belongs to after checking whether
	a highlight is allowed"
	(field checkHighlight: player)
		ifTrue: [field highlight.
			selected := true]! !

!AbaSphere methodsFor: 'accessing' stamp: 'pd 1/5/2007 19:45'!
field: aField
	field := aField! !

!AbaSphere methodsFor: 'accessing' stamp: 'pd 1/8/2007 22:22'!
getPlayer
	^ player! !

!AbaSphere methodsFor: 'accessing' stamp: 'pd 1/8/2007 22:24'!
setPlayer: anId 
	player := anId! !

!AbaSphere methodsFor: 'initialization' stamp: 'tfl 1/24/2007 17:21'!
initialize
	"initializes a sphere and connects it with a mouse event in
	order to select/deselect fields/spheres"
	super initialize.
	self extent: 40 @ 40;
		 left: 3;
		 top: 3;
		 borderWidth: 0;
	on: #mouseUp
	send: #aMouseUp:
	to: self.
	selected := false! !

!AbaSphere methodsFor: 'initialization' stamp: 'tfl 1/24/2007 19:24'!
makeGradient
	"give the spheres a round look, so they actually look like
	spheres"
	| fill |
	fill := GradientFillStyle ramp: {0.0 -> Color white. 0.2 -> self color muchLighter. 0.5 -> self color darker. 1.0 -> self color muchDarker}.
	fill
		origin: (self bounds topLeft translateBy: self bounds width // 3 @ (self bounds height // 3)).
	fill direction: self bounds width // 2 @ 0.
	fill radial: true.
	self fillStyle: fill! !

